/*
Copyright (C) 2006-2007 by Carlos H. P. da Silva <carloshp.silva@gmail.com>.

This file is part of LameScript 2.0.

    LameScript is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
// ====================================================================
// lstest.c - example code to test or show LameScript features
// ====================================================================

#include "..\lib\lscript.h"

/**
 * WARNING!WARNING!WARNING!WARNING!WARNING!
 * Some of these functions are just tests
 * I wrote for some LameScript internals,
 * NOT usage examples. Stick to the public
 * API as declared in the README.txt file to
 * avoid incompatibilities in the future!
 **/
void	test01(void)
{
	int		r, s;
	char	buf[10];
	int		count[HASHBUCKETS];
	int		total = 0;

	for (r = 0; r < HASHBUCKETS; r++)
	{
		count[r] = 0;
	}

	for (s = '0'; s <= 'z'; s++)
	{
		for (r = '0'; r <= 'z'; r++)
		{
			sprintf (buf, "%c%c\0", s, r);
			printf ("_getHashCode(\"%s\") = %02X\n", buf, _getHashCode(buf));
			count[_getHashCode(buf)]++;
			total++;
		}
	}

	printf ("\nResults for %d combinations:\n", total);
	for (r = 0; r < HASHBUCKETS; r++)
	{
		printf ("key #%d, %d occurrences.\n", r, count[r]);
	}
}

void	dumpHashTableItem(lshitem_t *item)
{
	printf ("name= \"%s\", ", (item->name != NULL ? item->name : "<null>"));
	printf ("content= %s, ", (item->item != NULL ? "OBJECT" : "<null>"));
	printf ("next= %s", (item->next != NULL ? "ITEM" : "<null>"));
}


void	dumpHashTableItems(lshash_t *hash)
{
	int			r;
	lshitem_t	*item;

	for (r = 0; r < HASHBUCKETS; r++)
	{
		printf ("Bucket #%d\n", r);
		item = hash->items[r];
		while (item->name)
		{
			printf ("\t[");
			dumpHashTableItem(item);
			printf ("]\n");
			item = (lshitem_t *)item->next;
		}
	}

	printf ("\n\n%d item(s).\n", hash->count);
}

void	test02(void)
{
	lshash_t	*hash = NULL;
	char		*buf1, *buf2;

	buf1 = malloc(64);
	sprintf (buf1, "Hello world\n");

	buf2 = malloc(64);
	sprintf (buf2, "Another stupid message");

	printf ("Creating a new hash table...");
	hash = _newHashTable("myhashtable");
	printf (" OK.\nDumping hash table info:\n\n");
	printf ("hash table name: %s\n", hash->name);
	printf ("items details:\n");
	dumpHashTableItems(hash);
	printf ("\nAdding item \"item01\", hash code == %d\n", _setHashTableItem(hash, "item01", NULL));
	printf ("\nAdding item \"item1\", hash code == %d\n", _setHashTableItem(hash, "item1", buf1));
	printf ("\nAdding item \"message\", hash code == %d\n", _setHashTableItem(hash, "message", NULL));
	printf ("\nAdding item \"foobar\", hash code == %d\n", _setHashTableItem(hash, "foobar", NULL));
	printf ("Items details:\n");
	dumpHashTableItems(hash);
	printf ("\nReplacing the \"foobar\" item... returned %d\n", _setHashTableItem(hash, "foobar", buf2));
	printf ("Items details:\n");
	dumpHashTableItems(hash);
	printf ("Recovering contents for item \"item1\" == %s\n", _getHashTableItem (hash, "item1"));
	printf ("Recovering contents for item \"foobar\" == %s\n", _getHashTableItem (hash, "foobar"));
	printf ("Removing item \"item01\" == %d\n", _delHashTableItem(hash, "item01"));
	printf ("Removing item \"item1\" == %d\n", _delHashTableItem(hash, "item1"));
	printf ("Items details:\n");
	dumpHashTableItems(hash);
	printf ("Total number of items: %d\n", _getHashTableCount(hash));
	printf ("Clearing hash table...\n");
	_clearHashTable(hash);
	printf ("Items details:\n");
	dumpHashTableItems(hash);
	printf ("Total number of items: %d\n", _getHashTableCount(hash));
}

void	test03(void)
{
	lshash_t	*hash = _newHashTable ("test03hash");
	char		*buf, *buf2;
	int			r;

	for (r = 0; r < 10; r++)
	{
		buf = malloc (20);
		buf2 = malloc (20);
		sprintf (buf, "name%02d\0", r);
		sprintf (buf2, "message%02d\0", r);
		_setHashTableItem (hash, buf, buf2);
	}

	dumpHashTableItems(hash);
	_setHashTableIterator (hash, "name00");
	while (_hasNextHashTableItem())
	{
		printf("%s", _nextHashTableItem());
		if (_hasNextHashTableItem())
		{
			printf (", ");
		}
		else
		{
			printf (".\n");
		}
	}

	_setHashTableIterator (hash, "name05");
	while (_hasNextHashTableItem())
	{
		printf("%s", _nextHashTableItem());
		if (_hasNextHashTableItem())
		{
			printf (", ");
		}
		else
		{
			printf (".\n");
		}
	}
}

extern	lshash_t	*scripts;
void	test04 (void)
{
	lshash_t	*script, *section, *key;
	printf ("checking initial status...\n");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to access without initialization...\n");
	LS_setPath ("$foobarscript:foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	LS_init();
	printf ("Status after initialization...\n");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to access without scripts...\n");
	LS_setPath ("$foobarscript:foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Adds a fake script...\n");
	script = _newHashTable ("foobarscript");
	_setHashTableItem (scripts, "foobarscript", script);
	LS_setPath ("$foobarscript:foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Adds a fake section...\n");
	section = _newHashTable("foobarsection");
	_setHashTableItem (script, "foobarsection", section);
	LS_setPath ("$foobarscript:foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Adds a fake key...\n");
	key = _newHashTable ("foobarkey");
	_setHashTableItem (section, "foobarkey", key);
	LS_setPath ("$foobarscript:foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Full path test complete\nTry to set with \"$foobarscript\"\n");
	LS_setPath ("$foobarscript");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to set with \"$foobarscript:foobarsection\"\n");
	LS_setPath ("$foobarscript:foobarsection");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to set with \":foobarsection/foobarkey\"\n");
	LS_setPath (":foobarsection/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to set with \"/foobarkey\"\n");
	LS_setPath ("/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("relative path tests complete\n");
	printf ("Try to use corrupted paths:\n");
	printf ("Try to set with \"$foobarscript/foobarkey\"\n");
	LS_setPath ("$foobarscript/foobarkey");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
	printf ("Try to set with \"/foobarkey$foobarscript\"\n");
	LS_setPath ("/foobarkey$foobarscript");
	printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	printf ("Current path is: \"%s\"\n", LS_getPath());
}

void	test05 (void)
{
	int		scr, sec, key, val;
	char	buf[64];


	printf ("Init...\n");
	LS_init();
	printf ("Creating script1...\n");
	LS_setScript ("script1");
	printf ("Creating section1...\n");
	LS_setSection ("section1");
	printf ("Creating key1...\n");
	LS_setKey ("key1");
	printf ("Current path is \"%s\"\n", LS_getPath());
	printf ("Adds the values list...\n");
	LS_addValue ("value00");
	LS_addValue ("value01");
	LS_addValue ("value02");
	LS_addValue ("value03");
	LS_addValue ("value04");
	LS_addValue ("value05");
	LS_addValue ("value06");
	LS_addValue ("value07");
	LS_addValue ("value08");
	LS_addValue ("value09");
	printf ("Number of scripts: %d\n", LS_getScriptCount());
	printf ("Number of sections in the current script: %d\n", LS_getSectionCount());
	printf ("Number of keys in the current section: %d\n", LS_getKeyCount());
	printf ("Number of values n the current key value list: %d\n", LS_getValueCount());
	printf ("Saving...\n");
	LS_saveScript ("savetest.ls");

	printf ("Testing multiple big script creation...\n");

	for (scr = 0; scr < 25; scr++)
	{
		sprintf (buf, "script%02d\0", scr);
		LS_setScript (buf);

		for (sec = 0; sec < 25; sec++)
		{
			sprintf (buf, "section%02d%02d\0", scr, sec);
			LS_setSection (buf);
			for (key = 0; key < 25; key++)
			{
				sprintf (buf, "key%02d%02d%02d\0", scr, sec, key);
				LS_setKey (buf);

				for (val = 0; val < 25; val++)
				{
					sprintf (buf, "val%02d%02d%02d%02d\0", scr, sec, key, val);
					LS_addValue (buf);
				}
			}
		}

		sprintf (buf, "savetest%02d.ls", scr);
		printf ("Saving script: %s\n", buf);
		LS_saveScript (buf);
	}

	printf ("Done.\n\n");
}

void	test06 (void)
{
	LS_init ();

	printf ("Loading script01.ls...\n");
	if (!LS_loadScript ("script01.ls", "svscript01"))
	{
		printf ("Info: %d sections\n", LS_getSectionCount());
		printf ("Deleting section0000...\n");
		LS_delSection ("section0000");
        printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
		printf ("Deleting section0099...\n");
		LS_delSection ("section0099");
        printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
		printf ("Info: %d sections\n", LS_getSectionCount());
		printf ("Saving svscript01.ls...\n");
		LS_saveScript ("svscript01.ls");
        printf ("LS_getStatus == 0x%04X\n", LS_getStatus());
	}
	else
	{
		printf ("Failed to load script01.ls!\n");
	}
}

void	test07 (void)
{
	int		scr, sec, key;

	LS_init();

	printf ("Testing LS_setPath()...\n");

	for (scr = 0; scr < 10; scr++)
	{
		for (sec = 0; sec < 25; sec++)
		{
			for (key= 0; key < 100; key++)
			{
				LS_setPath ("$script%02d:section%02d/key%02d", scr, sec, key);
			}
		}
	}
}

int main (void)
{
    printf ("test01 start:\n");
	// tests _getHashCode()
	test01();

    printf ("test01 end.\ntest02 start:\n");
	// tests _newHashTable(),
	// _setHashTableItem(),
	// _getHashTableItem(),
	// _delHashTableItem(),
	// _getHashTableCount(),
	// _clearHashTable()
	test02();

    printf ("test02 end.\ntest03 start:\n");
	// tests _setHashTableIterator(),
	// _hasNextHashTableItem(),
	// _nextHashTableItem()
	test03();

    printf ("test03 end.\ntest04 start:\n");
	// tests LS_init(),
	// LS_getStatus(),
	// LS_setPath(),
	// LS_getPath()
	test04();

    printf ("test04 end.\ntest05 start:\n");
	// LS_Init(),
	// LS_setScript(),
	// LS_setSection(),
	// LS_setKey(),
	// LS_addValue(),
	// LS_saveScript(),
	// LS_saveScriptFile()
	test05();

    printf ("test05 end.\ntest06 start:\n");
	// LS_init(),
	// LS_loadScript(),
	// LS_saveScript()
	test06();

    printf ("test06 end.\ntest07 start:\n");
	// LS_setPath(...)
	test07();

    printf ("test07 end.\n");
	return (0);
}